@@ -5,6 +5,7 @@
 from __future__ import absolute_import, division
 import argparse
 import collections
+import math
 import multiprocessing
 import os
 import sys
@@ -1252,6 +1253,41 @@ P_import_seg.add_argument('-d', '--output-dir', default='.',
 P_import_seg.set_defaults(func=_cmd_import_seg)
 
 
+# import-theta ---------------------------------------------------------------
+
+def _cmd_import_theta(args):
+    """Convert THetA output to a BED-like, CNVkit-like tabular format.
+
+    Equivalently, use the THetA results file to convert CNVkit .cns segments to
+    integer copy number calls.
+    """
+    tumor_segs = CNA.read(args.tumor_cns)
+    theta = importers.parse_theta_results(args.theta_results)
+    for i, copies in enumerate(theta['C']):
+        # Replace segment values with these integers
+        # Drop any segments where the C value is None
+        new_segs = []
+        for seg, ncop in zip(tumor_segs.copy(), copies):
+            if ncop is None:
+                continue
+            seg["coverage"] = math.log((ncop or 0.5) / args.ploidy, 2)
+            new_segs.append(seg)
+        new_cns = tumor_segs.to_rows(new_segs)
+        new_cns.write(os.path.join(args.output_dir,
+                                   "%s-%d.cns" % (tumor_segs.sample_id, i + 1)))
+
+
+P_import_theta = AP_subparsers.add_parser('import-theta',
+        help=_cmd_import_theta.__doc__)
+P_import_theta.add_argument("tumor_cns")
+P_import_theta.add_argument("theta_results")
+P_import_theta.add_argument("--ploidy", type=int, default=2,
+        help="Ploidy of the sample cells.")
+P_import_theta.add_argument('-d', '--output-dir', default='.',
+        help="Output directory name.")
+P_import_theta.set_defaults(func=_cmd_import_theta)
+
+
 # export ----------------------------------------------------------------------
 
 def _cmd_export(args):
@@ -1263,6 +1299,22 @@ def _cmd_export(args):
         # Special case: Nexus "basic" can only represent 1 sample
         assert len(args.filenames) == 1
         outheader, outrows = export.export_nexus_basic(args.filenames[0])
+    elif args.filetype == 'freebayes':
+        # Special case: FreeBayes --cnv-map could represent multiple samples,
+        # but the exporter currently doesn't handle that.
+        assert len(args.filenames) == 1, \
+                "FreeBayes export requires one segmentation file (.cns)"
+        outheader, outrows = export.export_freebayes(args.filenames[0], args)
+    elif args.filetype == 'theta':
+        # Special case: THetA takes tumor .cns and normal .cnr or reference.cnn
+        assert len(args.filenames) == 2, \
+                """THetA export requires two input files: (1) tumor-sample
+                segmentation from CNVkit (.cns), and (2) normal-sample bin-level
+                copy ratios (.cnr), or a corresponding reference (.cnn)."""
+        tumor, reference = args.filenames
+        outheader, outrows = export.export_theta(tumor, reference)
+        # if not args.output:
+        #     args.output = tumor_segs.sample_id + ".input"
     else:
         sample_ids = list(map(core.fbase, args.filenames))
         rows = export.merge_samples(args.filenames)
@@ -1277,15 +1329,43 @@ P_export.add_argument('filetype', choices=export.EXPORT_FORMATS,
                 'jtv' (Java TreeView),
                 'cdt' (Java TreeView again),
                 'seg' (IGV and GenePattern),
-                'nexus-basic' (Nexus Copy Number)
+                'nexus-basic' (Nexus Copy Number),
+                'freebayes' (FreeBayes --cnv-map input file),
+                'theta' (THetA2 input file, *.input)
                 """)
                 # 'multi' (Nexus Copy Number "multi1")
                 # 'gct' (GenePattern).
 P_export.add_argument('filenames', nargs='+', metavar="filename(s)",
-        help="""Processed sample coverage data file(s) (*.cnr), the output
-                of the 'fix' sub-command.""")
+        help="""Processed sample coverage data file(s) (*.cnr or *.cns), the
+                output of the 'fix' sub-command.""")
 P_export.add_argument('-o', '--output',
         help="Output file name.")
+
+P_export_freebayes = P_export.add_argument_group(
+    """FreeBayes export options to generate the FreeBayes --cnv-map input file.
+
+    Input is: a segmentation file (.cns). This may be imported from THetA to
+    account for normal-cell contamination and subclonal tumor cell population
+    structure; otherwise, the --purity argument provides a simpler adjustment.
+    """)
+P_export_freebayes.add_argument("-n", "--name",
+        help="Sample name, as FreeBayes should see it.")
+P_export_freebayes.add_argument("--ploidy", type=int, default=2,
+        help="Ploidy of the sample cells.")
+P_export_freebayes.add_argument("--purity", type=float,
+        help="Estimated tumor cell purity or cellularity.")
+P_export_freebayes.add_argument("-g", "--gender",
+        choices=('m', 'male', 'f', 'female'),
+        help="""Specify the sample's gender as male or female. (Otherwise
+                guessed from chrX copy number).""")
+P_export_freebayes.add_argument("-y", "--male-normal", action="store_true",
+        help="""Was a male-normal reference used?  If so, expect half ploidy on
+                chrX and chrY; otherwise, only chrY has half ploidy.  In CNVkit,
+                if a male-normal copy number reference was used, the "neutral"
+                copy number (ploidy) of chrX is 1; chrY is haploid for either
+                gender reference.""")
+
+
 P_export.set_defaults(func=_cmd_export)
 
 
