@@ -230,8 +230,7 @@ def parse_bed_track(line):
             key, val = field.split('=', 1)
             if key == 'name':
                 return val
-    else:
-        raise ValueError("No name defined for this track")
+    raise ValueError("No name defined for this track")
 
 
 def group_bed_tracks(bedfile):
@@ -435,16 +434,15 @@ def ensure_bam_sorted(bam_fname, by_name=False, span=50):
         getter = lambda read: read.qname
     else:
         # Compare read locations
-        getter = lambda read: read.pos
+        getter = lambda read: (read.tid, read.pos)
 
     # TODO - repeat at 50%, ~99% through the BAM
     bam = pysam.Samfile(bam_fname, 'rb')
     last_read = None
     for read in islice(bam, span):
         read_value = getter(read)
-        if last_read is not None:
-            if read_value < last_read:
-                return False
+        if last_read is not None and read_value < last_read:
+            return False
         last_read = read_value
 
     return True
